library ieee;
use ieee.std_logic_1164.ALL;

library UNISIM;
use UNISIM.VComponents.all;

library work ;
use work.logi_wishbone_pack.all ;
use work.logi_wishbone_peripherals_pack.all ;


entity logi_face_lite is
	port(
		OSC_FPGA : in std_logic ;
		MOSI :  in std_logic;
		MISO :   out  std_logic;
		SS :  in std_logic;
		SCK :  in std_logic;
		LED :   out  std_logic_vector((2-1) downto 0);
		PMOD1 :   inout  std_logic_vector((8-1) downto 0);
		PMOD2 :   inout  std_logic_vector((8-1) downto 0);
		PMOD3 :   inout  std_logic_vector((8-1) downto 0);
		PMOD4 :   inout  std_logic_vector((8-1) downto 0);
		ARD :   inout  std_logic_vector((6-1) downto 0);
		PB :  in std_logic_vector((2-1) downto 0);
		SW :  in std_logic_vector((2-1) downto 0)	
	);
end logi_face_lite;

architecture structural of logi_face_lite is
	
	type wishbone_bus is
	record
		address : std_logic_vector(15 downto 0);
		writedata : std_logic_vector(15 downto 0);
		readdata : std_logic_vector(15 downto 0);
		cycle: std_logic;
		write : std_logic;
		strobe : std_logic;
		ack : std_logic;
	end record;

	signal gls_clk, gls_reset, clk_locked, osc_buff, clkfb : std_logic ;

	signal Master_0_wbm_Intercon_0_wbs_0 : wishbone_bus;
	signal top_MOSI_Master_0_mosi_0 : std_logic;
	signal top_SS_Master_0_ss_0 : std_logic;
	signal top_SCK_Master_0_sck_0 : std_logic;
	signal Master_0_miso_top_MISO_0 : std_logic;
	signal Intercon_0_wbm_PWM_0_wbs_0 : wishbone_bus;
	signal Intercon_0_wbm_SERVO_0_wbs_0 : wishbone_bus;
	signal Intercon_0_wbm_MAT_0_wbs_0 : wishbone_bus;
	signal Intercon_0_wbm_MIC_0_wbs_0 : wishbone_bus;
	signal SERVO_0_servos_top_PMOD1_0 : std_logic_vector((4-1) downto 0);
	signal MAT_0_dout_top_PMOD2_0 : std_logic;
	signal MAT_0_sclk_top_PMOD2_0 : std_logic;
	signal MAT_0_load_top_PMOD2_0 : std_logic;
	signal MIC_0_ss_top_PMOD3_0 : std_logic;
	signal MIC_0_sck_top_PMOD3_0 : std_logic;
	signal top_PMOD3_MIC_0_miso_0 : std_logic;
	signal PWM_0_pwm_out_top_PMOD4_0 : std_logic_vector((4-1) downto 0);

begin


gls_reset <= (NOT clk_locked); -- system reset while clock not locked



Master_0 : spi_wishbone_wrapper
-- no generics
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	mosi =>  top_MOSI_Master_0_mosi_0,

	miso =>  Master_0_miso_top_MISO_0,

	ss =>  top_SS_Master_0_ss_0,

	sck =>  top_SCK_Master_0_sck_0,

	wbm_address =>  Master_0_wbm_Intercon_0_wbs_0.address,
	wbm_writedata =>  Master_0_wbm_Intercon_0_wbs_0.writedata,
	wbm_readdata =>  Master_0_wbm_Intercon_0_wbs_0.readdata,
	wbm_cycle =>  Master_0_wbm_Intercon_0_wbs_0.cycle,
	wbm_strobe =>  Master_0_wbm_Intercon_0_wbs_0.strobe,
	wbm_write =>  Master_0_wbm_Intercon_0_wbs_0.write,
	wbm_ack =>  Master_0_wbm_Intercon_0_wbs_0.ack	
);

Intercon_0 : wishbone_intercon
generic map(
memory_map => ("000000000001XXXX", "000000000010XXXX", "0000000000000XXX", "00000001XXXXXXXX")
)
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	wbs_address =>  Master_0_wbm_Intercon_0_wbs_0.address,
	wbs_writedata =>  Master_0_wbm_Intercon_0_wbs_0.writedata,
	wbs_readdata =>  Master_0_wbm_Intercon_0_wbs_0.readdata,
	wbs_cycle =>  Master_0_wbm_Intercon_0_wbs_0.cycle,
	wbs_strobe =>  Master_0_wbm_Intercon_0_wbs_0.strobe,
	wbs_write =>  Master_0_wbm_Intercon_0_wbs_0.write,
	wbs_ack =>  Master_0_wbm_Intercon_0_wbs_0.ack,

	wbm_address(0) =>  Intercon_0_wbm_PWM_0_wbs_0.address,
		wbm_address(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.address,
		wbm_address(2) =>  Intercon_0_wbm_MAT_0_wbs_0.address,
		wbm_address(3) =>  Intercon_0_wbm_MIC_0_wbs_0.address,
	wbm_writedata(0) =>  Intercon_0_wbm_PWM_0_wbs_0.writedata,
		wbm_writedata(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.writedata,
		wbm_writedata(2) =>  Intercon_0_wbm_MAT_0_wbs_0.writedata,
		wbm_writedata(3) =>  Intercon_0_wbm_MIC_0_wbs_0.writedata,
	wbm_readdata(0) =>  Intercon_0_wbm_PWM_0_wbs_0.readdata,
		wbm_readdata(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.readdata,
		wbm_readdata(2) =>  Intercon_0_wbm_MAT_0_wbs_0.readdata,
		wbm_readdata(3) =>  Intercon_0_wbm_MIC_0_wbs_0.readdata,
	wbm_cycle(0) =>  Intercon_0_wbm_PWM_0_wbs_0.cycle,
		wbm_cycle(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.cycle,
		wbm_cycle(2) =>  Intercon_0_wbm_MAT_0_wbs_0.cycle,
		wbm_cycle(3) =>  Intercon_0_wbm_MIC_0_wbs_0.cycle,
	wbm_strobe(0) =>  Intercon_0_wbm_PWM_0_wbs_0.strobe,
		wbm_strobe(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.strobe,
		wbm_strobe(2) =>  Intercon_0_wbm_MAT_0_wbs_0.strobe,
		wbm_strobe(3) =>  Intercon_0_wbm_MIC_0_wbs_0.strobe,
	wbm_write(0) =>  Intercon_0_wbm_PWM_0_wbs_0.write,
		wbm_write(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.write,
		wbm_write(2) =>  Intercon_0_wbm_MAT_0_wbs_0.write,
		wbm_write(3) =>  Intercon_0_wbm_MIC_0_wbs_0.write,
	wbm_ack(0) =>  Intercon_0_wbm_PWM_0_wbs_0.ack,
		wbm_ack(1) =>  Intercon_0_wbm_SERVO_0_wbs_0.ack,
		wbm_ack(2) =>  Intercon_0_wbm_MAT_0_wbs_0.ack,
		wbm_ack(3) =>  Intercon_0_wbm_MIC_0_wbs_0.ack	
);

PWM_0 : wishbone_pwm
generic map(
nb_chan => 7
)
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	wbs_address =>  Intercon_0_wbm_PWM_0_wbs_0.address,
	wbs_writedata =>  Intercon_0_wbm_PWM_0_wbs_0.writedata,
	wbs_readdata =>  Intercon_0_wbm_PWM_0_wbs_0.readdata,
	wbs_cycle =>  Intercon_0_wbm_PWM_0_wbs_0.cycle,
	wbs_strobe =>  Intercon_0_wbm_PWM_0_wbs_0.strobe,
	wbs_write =>  Intercon_0_wbm_PWM_0_wbs_0.write,
	wbs_ack =>  Intercon_0_wbm_PWM_0_wbs_0.ack,

	pwm_out(3 downto 0) =>  PWM_0_pwm_out_top_PMOD4_0,
	pwm_out(6 downto 4) => open	
);

SERVO_0 : wishbone_servo
generic map(
nb_servos => 8
)
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	wbs_address =>  Intercon_0_wbm_SERVO_0_wbs_0.address,
	wbs_writedata =>  Intercon_0_wbm_SERVO_0_wbs_0.writedata,
	wbs_readdata =>  Intercon_0_wbm_SERVO_0_wbs_0.readdata,
	wbs_cycle =>  Intercon_0_wbm_SERVO_0_wbs_0.cycle,
	wbs_strobe =>  Intercon_0_wbm_SERVO_0_wbs_0.strobe,
	wbs_write =>  Intercon_0_wbm_SERVO_0_wbs_0.write,
	wbs_ack =>  Intercon_0_wbm_SERVO_0_wbs_0.ack,

	failsafe => '0',

	servos(3 downto 0) =>  SERVO_0_servos_top_PMOD1_0,
	servos(7 downto 4) => open	
);

MAT_0 : wishbone_max7219
generic map(
nb_device => 2
)
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	wbs_address =>  Intercon_0_wbm_MAT_0_wbs_0.address,
	wbs_writedata =>  Intercon_0_wbm_MAT_0_wbs_0.writedata,
	wbs_readdata =>  Intercon_0_wbm_MAT_0_wbs_0.readdata,
	wbs_cycle =>  Intercon_0_wbm_MAT_0_wbs_0.cycle,
	wbs_strobe =>  Intercon_0_wbm_MAT_0_wbs_0.strobe,
	wbs_write =>  Intercon_0_wbm_MAT_0_wbs_0.write,
	wbs_ack =>  Intercon_0_wbm_MAT_0_wbs_0.ack,

	dout =>  MAT_0_dout_top_PMOD2_0,

	sclk =>  MAT_0_sclk_top_PMOD2_0,

	load =>  MAT_0_load_top_PMOD2_0	
);

MIC_0 : wishbone_pmic
generic map(
sample_rate => 8000
)
port map(
	gls_clk => gls_clk, gls_reset => gls_reset,

	wbs_address =>  Intercon_0_wbm_MIC_0_wbs_0.address,
	wbs_writedata =>  Intercon_0_wbm_MIC_0_wbs_0.writedata,
	wbs_readdata =>  Intercon_0_wbm_MIC_0_wbs_0.readdata,
	wbs_cycle =>  Intercon_0_wbm_MIC_0_wbs_0.cycle,
	wbs_strobe =>  Intercon_0_wbm_MIC_0_wbs_0.strobe,
	wbs_write =>  Intercon_0_wbm_MIC_0_wbs_0.write,
	wbs_ack =>  Intercon_0_wbm_MIC_0_wbs_0.ack,

	ss =>  MIC_0_ss_top_PMOD3_0,

	sck =>  MIC_0_sck_top_PMOD3_0,

	miso =>  top_PMOD3_MIC_0_miso_0	
);



-- Connecting inputs
top_MOSI_Master_0_mosi_0 <= MOSI;
top_SS_Master_0_ss_0 <= SS;
top_SCK_Master_0_sck_0 <= SCK;
-- <= PB;
-- <= SW;

-- Connecting outputs
MISO <= Master_0_miso_top_MISO_0;
LED(1 downto 0) <= (others => 'Z');

-- Connecting inouts

PMOD1(3 downto 0) <= SERVO_0_servos_top_PMOD1_0;
--SERVO_0_servos_top_PMOD1_0 <= PMOD1(3 downto 0);


PMOD1(7 downto 4) <= (others => 'Z');
--(others => 'Z') <= PMOD1(7 downto 4);


PMOD2(0) <= MAT_0_dout_top_PMOD2_0;
--MAT_0_dout_top_PMOD2_0 <= PMOD2(0);


PMOD2(1) <= MAT_0_sclk_top_PMOD2_0;
--MAT_0_sclk_top_PMOD2_0 <= PMOD2(1);


PMOD2(2) <= MAT_0_load_top_PMOD2_0;
--MAT_0_load_top_PMOD2_0 <= PMOD2(2);


PMOD2(7 downto 3) <= (others => 'Z');
--(others => 'Z') <= PMOD2(7 downto 3);


PMOD3(0) <= MIC_0_ss_top_PMOD3_0;
--MIC_0_ss_top_PMOD3_0 <= PMOD3(0);


PMOD3(3) <= MIC_0_sck_top_PMOD3_0;
--MIC_0_sck_top_PMOD3_0 <= PMOD3(3);


--PMOD3(2) <= top_PMOD3_MIC_0_miso_0;
top_PMOD3_MIC_0_miso_0 <= PMOD3(2);


PMOD3(1) <= 'Z';
--'Z' <= PMOD3(1);


PMOD3(7 downto 4) <= (others => 'Z');
--(others => 'Z') <= PMOD3(7 downto 4);


PMOD4(3 downto 0) <= PWM_0_pwm_out_top_PMOD4_0;
--PWM_0_pwm_out_top_PMOD4_0 <= PMOD4(3 downto 0);


PMOD4(7 downto 4) <= (others => 'Z');
--(others => 'Z') <= PMOD4(7 downto 4);


ARD(5 downto 0) <= (others => 'Z');
--(others => 'Z') <= ARD(5 downto 0);






-- system clock generation

PLL_BASE_inst : PLL_BASE generic map (
      BANDWIDTH      => "OPTIMIZED",        -- "HIGH", "LOW" or "OPTIMIZED" 
      CLKFBOUT_MULT  => 16 ,                 -- Multiply value for all CLKOUT clock outputs (1-64)
      CLKFBOUT_PHASE => 0.0,                -- Phase offset in degrees of the clock feedback output (0.0-360.0).
      CLKIN_PERIOD   => 20.00,              -- Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
      -- CLKOUT0_DIVIDE - CLKOUT5_DIVIDE: Divide amount for CLKOUT# clock output (1-128)
      CLKOUT0_DIVIDE => 8,       CLKOUT1_DIVIDE => 1,
      CLKOUT2_DIVIDE => 1,       CLKOUT3_DIVIDE => 1,
      CLKOUT4_DIVIDE => 1,       CLKOUT5_DIVIDE => 1,
      -- CLKOUT0_DUTY_CYCLE - CLKOUT5_DUTY_CYCLE: Duty cycle for CLKOUT# clock output (0.01-0.99).
      CLKOUT0_DUTY_CYCLE => 0.5, CLKOUT1_DUTY_CYCLE => 0.5,
      CLKOUT2_DUTY_CYCLE => 0.5, CLKOUT3_DUTY_CYCLE => 0.5,
      CLKOUT4_DUTY_CYCLE => 0.5, CLKOUT5_DUTY_CYCLE => 0.5,
      -- CLKOUT0_PHASE - CLKOUT5_PHASE: Output phase relationship for CLKOUT# clock output (-360.0-360.0).
      CLKOUT0_PHASE => 0.0,      CLKOUT1_PHASE => 0.0, -- Capture clock
      CLKOUT2_PHASE => 0.0,      CLKOUT3_PHASE => 0.0,
      CLKOUT4_PHASE => 0.0,      CLKOUT5_PHASE => 0.0,
      
      CLK_FEEDBACK => "CLKFBOUT",           -- Clock source to drive CLKFBIN ("CLKFBOUT" or "CLKOUT0")
      COMPENSATION => "SYSTEM_SYNCHRONOUS", -- "SYSTEM_SYNCHRONOUS", "SOURCE_SYNCHRONOUS", "EXTERNAL" 
      DIVCLK_DIVIDE => 1,                   -- Division value for all output clocks (1-52)
      REF_JITTER => 0.1,                    -- Reference Clock Jitter in UI (0.000-0.999).
      RESET_ON_LOSS_OF_LOCK => FALSE        -- Must be set to FALSE
   ) port map (
      CLKFBOUT => clkfb, -- 1-bit output: PLL_BASE feedback output
      -- CLKOUT0 - CLKOUT5: 1-bit (each) output: Clock outputs
      CLKOUT0 => gls_clk,      CLKOUT1 => open,
      CLKOUT2 => open,      CLKOUT3 => open,
      CLKOUT4 => open,      CLKOUT5 => open,
      LOCKED  => clk_locked,  -- 1-bit output: PLL_BASE lock status output
      CLKFBIN => clkfb, -- 1-bit input: Feedback clock input
      CLKIN   => osc_buff,  -- 1-bit input: Clock input
      RST     => '0'    -- 1-bit input: Reset input
   );

    -- Buffering of clocks
	BUFG_1 : BUFG port map (O => osc_buff,    I => OSC_FPGA);

end structural ;
