import sys
sys.path.append("/home/pi/logi-tools/python/")

import logi, time, random, math, threading
import logi_hal

#buffer to display eye_ball
eye_ball = [0x3C, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E, 0x7E] #orignal
#eye_ball = [0x00, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x00]	# mirrored in teh y.
#eye_ball = [0x00, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x00]


#buffer to display blinkink eye
eye_blink = [0x00, 0x00, 0x3C, 0x7E, 0x7E, 0x3C, 0x00, 0x00]

#pupil pattern small and large
pupil_small = [0x03, 0x03]
large_pupil = [0x03, 0x0F, 0x0F, 0x03]

#helper function
def countOneBits(bits):
		count = 0
		masked = 0x00
		for i in range(0, 8):
			masked = bits & (1 << i)
			if masked != 0 :
				count = count + 1
		return count

#address map of logi-face, maybe subject to change, can export from skeleton
PWM_0=0x0010
SERVO_0=0x0020
MAT_0=0x0000
MIC_0=0x0100

def blinkLeds(face, period, nb, power):
	for i  in range(nb):
		face.setAllPwm(int(round(face.pwm_period*power)))
		time.sleep(period/2)
		face.setAllPwm(0)
		time.sleep(period/2)

def fadeLeds(face, period, nb, power):
	intens = 0
	for i  in range(nb):
		int(round(math.fabs(math.sin(intens)*0x0800)))
		intens = intens + 0.01
		face.setAllPwm(int(round(face.pwm_period*math.fabs(math.sin(intens)) )))
		time.sleep(period)
	face.setAllPwm(0)

def cycleLeds(face, period, nb, power):
	for i in range(nb):
		face.setPwm(i%3, int(round(face.pwm_period*power)))
		face.setPwm((i+1)%3, 0)
		face.setPwm((i+2)%3, 0)
		time.sleep(period)
	face.setAllPwm(0)

#LOGI-Face main class
class LogiFace(object):

	servo_base_address = [SERVO_0, SERVO_0+1, SERVO_0+2, SERVO_0+3]
	mat_base_address = MAT_0
	pwm_base_address = PWM_0
	pwm_period = 0x0800	
	led_delay = 5
	
	#attitudes are group as follow dictionnary key is a smiley
	# each attitude is organized as : 
	# eyex, eyey, eye_type, eyebrow_right_angle, eyebrow_left_angle, mouth_right_angle, mouth_left_angle
	# led animation function and parameters if any
	attitude = {	
		':|' : [0, 0, 0, 0.0, 0.0, 0.0, 0.0],
		':)' : [0, 0, 0, 0.0, 0.0, 30.0, -30.0, blinkLeds, 0.1, led_delay, 0.1], 
		':(' : [0, 0, 0, 0.0, 0.0, -30.0, 30.0, fadeLeds, 0.1, led_delay, 0.1], 
		'8$' : [0, 0, 1, 30.0, -30.0, 30.0, 30.0, cycleLeds, 0.1, led_delay, 0.1]}

	def __init__(self):
		logi_hal.setPWMDivider(self.pwm_base_address, 4)
		logi_hal.setPWMPeriod(self.pwm_base_address, 0x0800)
		self.pwm_period = 0x0800
		self.mic_offset = 800 
		self.led_delay = 10
	
		
	def getSoundLevel(self):
		i = 0
		mean = 0
		new_offset = 0 ;
		logi.logiWrite(MIC_0, (0x00, 0x00)) # reset fifo
		while i < 256:
			samples = logi.logiRead(MIC_0, 256);
			for k in range(128):
				if (samples[(k*2)+1] & 0x80) != 0:
					i  = i + 1
					new_offset =  new_offset + ((samples[(k*2)+1] & 0x0F) << 8) + samples[(k*2)]
					sample_level = ((samples[(k*2)+1] & 0x0F) << 8) + samples[(k*2)] - self.mic_offset
					mean = mean + abs(sample_level)
		mean = (mean/256.0)
		self.mic_offset = new_offset/256.0
		return mean 	

	def getEyeBuffer(self, posx, posy, pupil=pupil_small):
		pupil_buffer = tuple()
		dimy = len(pupil)
		fp = 4 - (dimy/2) + posy
		lp = 4 + (dimy/2) + posy
		for i in range(0, 8):
			if i >= fp and i < lp:
				dimx = countOneBits(pupil[i-fp])
				decx = 4 - (dimx/2) + posx
				pupil_buffer = pupil_buffer + (eye_ball[i] & (~(pupil[i-fp] << (decx) )), ) 
			else:
				pupil_buffer = pupil_buffer + (eye_ball[i], ) 
		return pupil_buffer
	#initialize max7219 inner registers
	def initEye(self):	
		logi.logiWrite(self.mat_base_address, (0x01,0x0C, 0x01, 0x0C))
		logi.logiWrite(self.mat_base_address, (0xF1,0x0A, 0xF1, 0x0A))
		logi.logiWrite(self.mat_base_address, (0x00,0x0F, 0x00, 0x0F))
		logi.logiWrite(self.mat_base_address, (0x00,0x09, 0x00, 0x09))
		logi.logiWrite(self.mat_base_address, (0x07,0x0B, 0x07, 0x0B))

	def writeEye(self, eye_buffer):
		long_buffer = tuple()
		for k in range(0, 8):
			long_buffer = long_buffer +(eye_buffer[k],k+1, eye_buffer[k], k+1,)
		logi.logiWrite(self.mat_base_address, long_buffer[0:16])
		logi.logiWrite(self.mat_base_address, long_buffer[16:32])

	def setPwm(self, index, val):
		logi_hal.setPWMPulse(self.pwm_base_address, index, val)

	def setAllPwm(self, val):
		logi.logiWrite(self.pwm_base_address+2, ((val & 0x00FF), val >> 8, (val & 0x00FF), val >> 8, (val & 0x00FF), val >> 8))


	# apply the given attitude based on smiley key
	def writeAttitude(self, smiley):
		att = self.attitude[smiley]
		if att[2] == 0:
			self.writeEye(self.getEyeBuffer(att[0], att[1]))
		else:
			self.writeEye(self.getEyeBuffer(att[0], att[1], large_pupil))
		logi_hal.setServoAngle(self.servo_base_address[0], 0, att[3])
		logi_hal.setServoAngle(self.servo_base_address[0], 1, att[4])
		logi_hal.setServoAngle(self.servo_base_address[0], 2, att[5])
		logi_hal.setServoAngle(self.servo_base_address[0], 3 , att[6])
		if len(att) > 10:
			att[7](self, att[8], att[9], att[10])


def toggleEyes():
	face.initEye()
	face.writeAttitude(':)')
	face.writeAttitude(':(')
	face.writeAttitude('8$')
	face.writeAttitude(':|')
	

if __name__ == "__main__":
	face = LogiFace()
	face.initEye()
	#face.writeAttitude(':)')
	#face.writeAttitude(':(')
	#face.writeAttitude('8$')
	#face.writeAttitude(':|')
	#cycleLeds(face,0.1, 20, 0.01)
	intens = 0 
	inc = 1
	
	SOUND_THRESHOLD = 150
	SOUND_DELAY = 2
	TIME_VAL = 800
	
	toggleEyes()
	time.sleep(0.01)	#wait for noise to settle
	while True:
		#toggleEyes()
		intens =  face.getSoundLevel()
		print intens
		if intens > SOUND_THRESHOLD :
			print "I Think I heard something!?\nI'm Scared..."
			tick = 0
			face.setPwm(0, 0x200)
			face.setPwm(1, 0x200)
			face.setPwm(2, 0x200)
			face.initEye()
			face.writeAttitude('8$')
			#face.writeAttitude(':|')
			face.initEye()
			time.sleep(SOUND_DELAY)
			face.writeAttitude(':|')
			time.sleep(0.1)	#delay the for noise to settle
			#while face.getSoundLevel() > SOUND_THRESHOLD :
			#	time.sleep(5)
			#continue	
		

		face.setPwm(0, int(round(intens/100)*face.pwm_period))
		face.setPwm(1, int(round(intens/100)*face.pwm_period))
		face.setPwm(2, int(round(intens/100)*face.pwm_period))

		
		if tick > TIME_VAL :
			print "I'm Bored!!!"
			toggleEyes()
			tick = 0
		
		
		tick = tick + 1
		#time.sleep(0.01)	#wait for noise to settle
